%#										
%#  function [T3, D] = bagged_clustering_kmeans(options);
%#										
%#  AIM: 	    Performs bagged Kmeans clustering
%#              
%#
%#  INPUT:		options.data                 - the data
%#              options.labels               - labels for the variables
%#              options.numberofbootstraps   - predefined number of bootstrap samples
%#              options.numberclusters       - predefined number of clusters
%#              options.kleur                - 0 = make all plots in colors
%#              options.histoutput           - 1 = plot histograms
%#              options.plot                 - 1 = plot collected histograms as heatplot
%#	 							
%#	 							
%#  OUTPUT:	    T3  - final partioning
%#              D   - distances
%#
%#										
%#  AUTHOR: 	Dr. J.A. Hageman
%#	            Copyright(c) 2005 for UvA
%#              Biosystems Data Analysis
%#              Swammerdam Institute for Life Sciences (SILS)
%#              Nieuwe Achtergracht 166
%#              1018 WV Amsterdam, NL
%#   										
%#  VERSION:    1.0 (08/12/2005)
%#																	
%#  USES:       kmeans, hist from Matlab, Mathworks
%#              


function [T3, D] = bagged_clustering_kmeans(options)

numberofexperiments=size(options.data,1);
numberofmetabolites=size(options.data,1);

fprintf('Resampling: ');
subcluster=zeros(numberofmetabolites, numberofmetabolites, options.numberofbootstraps);

for bootstrapsample=1:options.numberofbootstraps
    subset=random_number(1, numberofexperiments, numberofexperiments, 1);
    datasub=options.data(subset, :);
    
    T = kmeans(datasub, options.numberclusters, 'Distance', 'sqeuclidean', 'Display', 'off', 'Replicates', 5, 'EmptyAction', 'singleton');
    for h=1:options.numberclusters
        k=find(T==h);
        for j=1:size(k,1)
            for i=1:size(k,1)
                subcluster(subset(k(j)), subset(k(i)), bootstrapsample)=h;
            end
        end
    end

    if mod(bootstrapsample, 50) == 0;
        fprintf('%3i ', bootstrapsample);
    end
end

fprintf('\n');

results_hist=zeros(numberofmetabolites, numberofmetabolites);

for k=1:numberofmetabolites
    S=squeeze(subcluster(k,:,:));

    [I J]=find(S~=0);
    numofbins=max(I)-min(I)+1;
    tempm=hist(sort(I), numofbins);
    m=[zeros(1,max(I)-size(tempm,2)) tempm zeros(1, numberofmetabolites-max(I))];
    results_hist(k, 1:numberofmetabolites) = m./m(k);

    if (options.histoutput)
        figure
        set(gcf, 'Position', [360 70 660 870])
        [h1, h2]=hist(sort(I), numberofmetabolites);
        q=barh(h2-0.5, h1);
        set(q, 'EdgeColor', 'Black');
        set(q, 'FaceColor', 'Black');
        set(gca, 'Ylim', [-0.5, (numberofmetabolites )+ 0.5], 'YTick', 1:numberofmetabolites, 'YTickLabel', options.labels, 'Box', 'off', 'FontSize', 9);
        title(['Mebtabolite ' options.labels{k} ' clusters with']);
        xlabel('Frequency');        
        pause
    end
end


[T3, C, D] = kmeans(results_hist', options.numberclusters, 'Replicates', 10, 'Display', 'off', 'EmptyAction', 'singleton');

for i=1:size(unique(T3),1)
    aantal(i)=size(find(T3==i),1);
end
[P,Q]=sort(aantal, 2, 'descend');

for i=1:size(unique(T3),1)
    T3(find(T3==Q(i)))=100+i;
end
T3=T3-100;

if (options.plot==1)

    %final clustering on histogram data
    h(1)= figure(...
        'Color',[1 1 1],...
        'InvertHardcopy','off',...
        'PaperUnits','centimeters',...
        'PaperOrientation','landscape',...
        'PaperPosition',[0.6345 0.6345 28.41 19.72],...
        'PaperSize',[29.68 20.98],...
        'PaperType','a4letter');
    set(h(1), 'Position', [10 300 1260 615]);

    subplot(2,1,1)
    hold on
    title('Unordered heatplot from metabolite histograms');
    axis off
    b=results_hist;
    b2=[b b(:,1)]; b3=[b2; b2(1,:)];
    imagesc(b3);
    colorbar;
    set(gca, 'XTick', []);
    set(gca, 'YTick', []);

    for i=1:size(options.labels,2)
        text(i+0.5, 0, options.labels{i}, 'Rotation', 90, 'FontSize', 7, 'HorizontalAlignment', 'right');
        text(0,i+0.5, options.labels{i}, 'FontSize', 7, 'HorizontalAlignment', 'right');
    end
    
    [I neworder]=sort(T3);

    %rearrange metabolites to obtain clearer heatplot
    fprintf('Reordering\n');
    options.labels=options.labels(neworder);
    results_hist=results_hist(:, neworder);
    results_hist=results_hist(neworder, :);

    subplot(2,1,2)
    hold on
    %show reordered heat plot
    title('Ordered heatplot from metabolite histograms');
    axis off
    
    %color or black and white?
    if (options.kleur~=0)
        map2=[1:-1/255:0; 1:-1/255:0; 1:-1/255:0;]';
        colormap(map2);
    end
    
    b=results_hist;
    b2=[b b(:,1)]; b3=[b2; b2(1,:)];
    imagesc(b3);
    colorbar;
    set(gca, 'XTick', []);
    set(gca, 'YTick', []);

    for i=1:size(options.labels,2)
        text(i+0.5, 0, options.labels{i}, 'Rotation', 90, 'FontSize', 7, 'HorizontalAlignment', 'right');
        text(0,i+0.5, options.labels{i}, 'FontSize', 7, 'HorizontalAlignment', 'right');
    end
end


function getal=random_number(ondergrens, bovengrens, aantal, integer)
if nargin == 2;
    aantal=1;
end

if nargin < 2;
    error('Give lowerlimit and upperlimit. Syntax: random_number(lowerlimit, upperlimit, how_many)');
end

if (integer==1)
    getal = (rand(1, aantal) * (bovengrens-ondergrens+1) ) + ondergrens;
    getal = floor(getal);
else
    getal = (rand(1, aantal) * (bovengrens-ondergrens)) + ondergrens;
end

